@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,12 +23,20 @@
 #ifndef _IOKIT_IO_SCSI_PROTOCOL_INTERFACE_H_
 #define _IOKIT_IO_SCSI_PROTOCOL_INTERFACE_H_
 
-#include <IOKit/storage/IOStorageDeviceCharacteristics.h>
 
 //
+//	Includes
+//
+
+#include <IOKit/storage/IOStorageDeviceCharacteristics.h>
+
+
+//
+//	Constants
+//
+
 //	SCSI Device Characteristics - 	Defined between SCSI Application Layer and
 //									SCSI Protocol Layer only.
-//
 
 // This key is used to define SCSI Device Characteristics for a particular device.
 // It is the key for the dictionary containing the keys of characteristics. These keys
@@ -51,18 +59,30 @@
 #if defined(KERNEL) && defined(__cplusplus)
 
 
+// This key is used as the CFBundleIdentifier key for the
+// IOSCSIArchitectureModelFamily proper
+#define kCFBundleIdentifierKey					"CFBundleIdentifier"
+#define kIOSCSIArchitectureBundleIdentifierKey	"com.apple.iokit.IOSCSIArchitectureModelFamily"
+
+
+// General kernel headers
+#include <kern/thread.h>
+
+// General IOKit headers
 #include <IOKit/IOLib.h>
 #include <IOKit/IOService.h>
-
-#include <IOKit/scsi-commands/SCSITask.h>
 #include <IOKit/IOCommandGate.h>
-#include <kern/thread.h>
+
+// SCSI Architecture Model Family includes
+#include <IOKit/scsi-commands/SCSITask.h>
+
 
 // SCSI Protocol Interface Device Notification values
 enum
 {
 	kSCSIProtocolNotification_DeviceRemoved			= 0x69000010,
-	kSCSIProtocolNotification_VerifyDeviceState		= 0x69000020
+	kSCSIProtocolNotification_VerifyDeviceState		= 0x69000020,
+	kSCSIServicesNotification_ExclusivityChanged	= 0x69000030
 };
 
 
@@ -83,13 +103,13 @@ enum
 	// kSCSIProtocolFeature_ProtocolSpecificPolling:
 	// Used to determine if the SCSI Protocol Services Driver supports
 	// protocol specific polling for media. This is used for low-power
-	// polling specifically on ATAPI buses.
+	// polling specifically for ATAPI devices on ATA buses.
 	kSCSIProtocolFeature_ProtocolSpecificPolling			= 3,
 	
 	// kSCSIProtocolFeature_ProtocolSpecificSleepCommand:
 	// Used to determine if the SCSI Protocol Services Driver supports
 	// protocol specific sleep commands to a drive. This is used for
-	// sleeping drives specifically on ATAPI buses.
+	// sleeping drives specifically ATAPI devices on ATA buses.
 	kSCSIProtocolFeature_ProtocolSpecificSleepCommand		= 4,
 	
 	// kSCSIProtocolFeature_GetMaximumLogicalUnitNumber:
@@ -106,11 +126,36 @@ enum
 	// it will return true to appropriate query and return the block count 
 	// in the UInt32 pointer that is passed in as the serviceValue. 
 	kSCSIProtocolFeature_MaximumReadBlockTransferCount		= 6,
-	kSCSIProtocolFeature_MaximumWriteBlockTransferCount		= 7
+	kSCSIProtocolFeature_MaximumWriteBlockTransferCount		= 7,
+
+	// kSCSIProtocolFeature_MaximumReadTransferByteCount and 
+	// kSCSIProtocolFeature_MaximumWriteTransferByteCount: 
+	// If the SCSI Protocol Services Driver has a maximum byte
+	// count that can be transferred in either a read or write request,
+	// it will return true to appropriate query and return the byte count 
+	// in the UInt64 pointer that is passed in as the serviceValue. 
+	kSCSIProtocolFeature_MaximumReadTransferByteCount		= 8,
+	kSCSIProtocolFeature_MaximumWriteTransferByteCount		= 9,
+	
+	// kSCSIProtocolFeature_SubmitDefaultInquiryData: 
+	// If the SCSI Protocol Services driver needs any extra information to 
+	// make any negotiation settings from the standard INQUIRY data, this 
+	// will be called to set that appropriately. The serviceValue will
+	// point to a SCSICmd_INQUIRY_StandardData buffer. The size
+	// of the buffer depends on the SCSI Device Characteristics
+	// dictionary for the device or bus. If there is no
+	// kIOPropertySCSIInquiryLengthKey value set in the dictionary
+	// or if it doesn't exist, then the size of the data will be
+	// the size of the full amount of Inquiry retrieved from the device.
+	kSCSIProtocolFeature_SubmitDefaultInquiryData			= 10
 	
 };
 
 
+//
+//	Class Declaration
+//
+
 // This class defines the public SCSI Protocol Layer API for any class that
 // provides Protocol services or needs to provide the Protocol Service API
 // for passing service requests to a Protocol Service driver. 
@@ -118,23 +163,23 @@ class IOSCSIProtocolInterface : public IOService
 {
 	
 	OSDeclareAbstractStructors ( IOSCSIProtocolInterface )
-
+	
 public:
 	
 	virtual bool	start 	( IOService * provider );
 	virtual void	free 	( void );
 	
 	// ------ User Client Support ------
-
+	
 	// The GetUserClientExclusivityState method is called by the user client
 	// to determine if any user client is holding exclusive access at the
 	// current time.
 	virtual bool 		GetUserClientExclusivityState ( void );
-
+	
 	// The SetUserClientExclusivityState method is called by the user client
 	// to set the exclusive access mode.
 	virtual IOReturn	SetUserClientExclusivityState ( IOService * userClient, bool state );
-
+	
 	// ------ Power Management Support ------
 	
 	// The initialPowerStateForDomainState() method is called by the power manager
@@ -158,16 +203,17 @@ public:
 	// requests, it gives the driver a chance to block the thread until the
 	// device is ready.
 	virtual void 		CheckPowerState ( void );
-
+	
 	// -- Functionality available to the SCSI Application Layer	--
-    // The ExecuteCommand method will take a SCSI Task and transport
-    // it across the physical wire(s) to the device
-    virtual void    	ExecuteCommand ( SCSITaskIdentifier request ) = 0;
-    
-    // The AbortCommand method will abort the indicated SCSI Task,
-    // if it is possible and the task has not already completed.
-    virtual SCSIServiceResponse    AbortCommand ( SCSITaskIdentifier request ) = 0;
-    
+	// The ExecuteCommand method will take a SCSI Task and transport
+	// it across the physical wire(s) to the device
+	virtual void		ExecuteCommand ( SCSITaskIdentifier request ) = 0;
+	
+	// ************* Obsoleted Member Routine ****************
+	// The AbortCommand method is replaced by the AbortTask Management function and
+	// should no longer be called.
+	virtual SCSIServiceResponse		AbortCommand ( SCSITaskIdentifier request ) = 0;
+	
 	// The IsProtocolServiceSupported will return true if the specified
 	// feature is supported by the protocol layer.  If the service has a value that must be
 	// returned, it will be returned in the serviceValue output parameter.
@@ -183,9 +229,10 @@ public:
 								void * 					serviceValue ) = 0;
 	
 protected:
-    // Reserve space for future expansion.
-    struct IOSCSIProtocolInterfaceExpansionData { };
-    IOSCSIProtocolInterfaceExpansionData *fIOSCSIProtocolInterfaceReserved;
+	
+	// Reserve space for future expansion.
+	struct IOSCSIProtocolInterfaceExpansionData { };
+	IOSCSIProtocolInterfaceExpansionData * fIOSCSIProtocolInterfaceReserved;
 	
 	// ------ Power Management Support ------
 	
@@ -195,34 +242,34 @@ protected:
 	UInt32				fProposedPowerState;
 	bool				fPowerTransitionInProgress;
 	bool				fPowerAckInProgress;
-    bool				fPowerManagementInitialized;
-	    
-    // The InitializePowerManagement method is called to initialize power management.
+	bool				fPowerManagementInitialized;
+		
+	// The InitializePowerManagement method is called to initialize power management.
 	// The default implementation calls PMinit() and joinPMtree but doesn't register
 	// the device for power managment. That is left to other layers to do.	
 	virtual void		InitializePowerManagement ( IOService * provider );
-
+	
 	// The GetInitialPowerState method is called once, right after InitializePowerManagement()
 	// in order to determine what state the device is initially in at startup time (usually
 	// the highest power mode).
 	virtual UInt32		GetInitialPowerState ( void ) = 0;
-
-    // The current implementation of the Power Manager requires that leaf nodes
-    // are PMstop'ed first. Moreover, if you call PMstop on a parent node first
-    // it will result in a kernel panic. We use finalize to call PMstop so we get
-    // "top down" ordering. However, the termination thread also needs to be
-    // synced with the power thread so we have to wait for any outstanding power
-    // transitions to complete before processing the finalize.
-    virtual bool		finalize ( IOOptionBits options );
-
+	
+	// The current implementation of the Power Manager requires that leaf nodes
+	// are PMstop'ed first. Moreover, if you call PMstop on a parent node first
+	// it will result in a kernel panic. We use finalize to call PMstop so we get
+	// "top down" ordering. However, the termination thread also needs to be
+	// synced with the power thread so we have to wait for any outstanding power
+	// transitions to complete before processing the finalize.
+	virtual bool		finalize ( IOOptionBits options );
+	
 	// The sHandleSetPowerState method is a static function used as C->C++ glue
 	// for going behind the command gate.
 	static IOReturn 	sHandleSetPowerState ( IOSCSIProtocolInterface * self, UInt32 powerStateOrdinal );
-
-    // The sGetPowerTransistionInProgress method is a static function used as
-    // C->C++ glue for getting the command gate.
-    static bool sGetPowerTransistionInProgress ( IOSCSIProtocolInterface * 	self );
-   
+	
+	// The sGetPowerTransistionInProgress method is a static function used as
+	// C->C++ glue for getting the command gate.
+	static bool sGetPowerTransistionInProgress ( IOSCSIProtocolInterface * 	self );
+	
 	// The HandleSetPowerState method is called by the glue code and is on the
 	// serialized side of the command gate. This allows us to touch any member
 	// variables as necessary without any multi-threading issues.
@@ -244,33 +291,33 @@ protected:
 	// The sHandleCheckPowerState method is a static function used as C->C++ glue
 	// for going behind the command gate.
 	static void			sHandleCheckPowerState ( IOSCSIProtocolInterface * self );
-
+	
 	// The HandleCheckPowerState (void) method is on the serialized side of the command
 	// gate and can change member variables safely without multi-threading issues.
 	// It's main purpose is to call the superclass' HandleCheckPowerState ( UInt32 maxPowerState )
 	// with the max power state the class registered with.
 	virtual void		HandleCheckPowerState ( void ) = 0;
-
+	
 	// The HandleCheckPowerState ( UInt32 maxPowerState ) method is called by
 	// the subclasses and is passed the maxPowerState number given to the power
 	// manager at initialization time. This guarantees the threads block until that
 	// power state has been achieved.
 	void 				HandleCheckPowerState ( UInt32 maxPowerState );
-
+	
 	// The TicklePowerManager ( void ) method is called by CheckPowerState and
 	// sends an activity tickle to the power manager so that the idle timer is
 	// reset.
 	virtual void		TicklePowerManager ( void ) = 0;
-
+	
 	// The TicklePowerManager ( UInt32 maxPowerState ) method is a convenience
 	// function which can be called by subclasses in TicklePowerManager (void)
 	// in order to tell the power manager to reset idle timer or bring the device
 	// into the requested state. It returns whatever is returned by activityTickle
 	// (true if device is in the requested state, false if it is not).
 	bool				TicklePowerManager ( UInt32 maxPowerState );
-
+	
 	// ------ User Client Support ------
-
+	
 	bool				fUserClientExclusiveControlled;
 	IOService *			fUserClient;
 	
@@ -281,25 +328,45 @@ protected:
 	
 	virtual bool 		HandleGetUserClientExclusivityState ( void );
 	virtual IOReturn 	HandleSetUserClientExclusivityState ( IOService * userClient, bool state );
-
+	
+public:
+	
+	// ------- SCSI Architecture Model Task Management Functions ------
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 1 );
+	// The ABORT TASK function allows an SCSI Application Layer client to request 
+	virtual SCSIServiceResponse		AbortTask ( UInt8 theLogicalUnit, SCSITaggedTaskIdentifier theTag ) = 0;
+	
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 2 );
+	virtual SCSIServiceResponse		AbortTaskSet ( UInt8 theLogicalUnit ) = 0;
+	
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 3 );
+	virtual SCSIServiceResponse		ClearACA ( UInt8 theLogicalUnit ) = 0;
+	
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 4 );
+	virtual SCSIServiceResponse		ClearTaskSet ( UInt8 theLogicalUnit ) = 0;
+	
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 5 );
+	virtual SCSIServiceResponse		LogicalUnitReset ( UInt8 theLogicalUnit ) = 0;
+	
+	OSMetaClassDeclareReservedUsed ( IOSCSIProtocolInterface, 6 );
+	
+	// The TARGET RESET management function will cause the Protocol Services Driver
+	// to perform the reset
+	virtual SCSIServiceResponse		TargetReset ( void ) = 0;
+	
 private:
+	
 	// Space reserved for future expansion.
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 1 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 2 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 3 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 4 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 5 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 6 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 7 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 8 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 9 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 10 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 11 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 12 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 13 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 14 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 15 );
-    OSMetaClassDeclareReservedUnused( IOSCSIProtocolInterface, 16 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface,  7 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface,  8 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface,  9 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 10 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 11 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 12 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 13 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 14 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 15 );
+	OSMetaClassDeclareReservedUnused ( IOSCSIProtocolInterface, 16 );
 	
 };
 
