@@ -27,7 +27,7 @@
 
 // SCSI Architecture Model Family includes
 #include "SCSITask.h"
-
+#include "SCSITaskDefinition.h"
 
 //
 //	Macros
@@ -159,6 +159,8 @@ SCSITask::ResetForNewTask ( void )
 	require ( ( IsTaskActive ( ) == false ), ErrorExit );
 	
 	fTaskAttribute 					= kSCSITask_SIMPLE;
+	fTaskTagIdentifier				= kSCSIUntaggedTaskIdentifier;
+	
    	fTaskState 						= kSCSITaskState_NEW_TASK;
 	fTaskStatus						= kSCSITaskStatus_GOOD;
 	fLogicalUnitNumber				= 0;	
@@ -329,6 +331,28 @@ SCSITask::GetTaskAttribute ( void )
 	return fTaskAttribute;
 }
 
+//
+//	 SetTaggedTaskIdentifier - Sets the SCSITaggedTaskIdentifier.	   [PUBLIC]
+//
+
+bool
+SCSITask::SetTaggedTaskIdentifier ( SCSITaggedTaskIdentifier newTag )
+{
+	fTaskTagIdentifier = newTag;
+	return true;
+}
+
+
+//
+//	 GetTaggedTaskIdentifier - Gets the SCSITaggedTaskIdentifier. 	   [PUBLIC]
+//
+
+SCSITaggedTaskIdentifier 
+SCSITask::GetTaggedTaskIdentifier ( void )
+{
+	return fTaskTagIdentifier;
+}
+
 
 //
 //	 SetTaskState - Sets the SCSITaskState to the new value. 		   [PUBLIC]
