@@ -598,19 +598,20 @@ IOReturn
 IOBlockStorageServices::reportMaxReadTransfer ( UInt64 		blockSize,
 												UInt64 * 	max )
 {
-	
-	if ( fMaxReadBlocks == 0 )
-	{
-		*max = blockSize * 256;
-	}
-	
-	else
-	{
-		*max = blockSize * fMaxReadBlocks;
-	}
-	
-	return kIOReturnSuccess;
-	
+	return kIOReturnUnsupported;
+}
+
+
+//
+//	 reportMaxWriteTransfer - Reports maximum write transfer in bytes.
+//																	   [PUBLIC]
+//
+
+IOReturn
+IOBlockStorageServices::reportMaxWriteTransfer ( UInt64		blockSize,
+												 UInt64 * 	max )
+{
+	return kIOReturnUnsupported;
 }
 
 
@@ -629,31 +630,6 @@ IOBlockStorageServices::reportMaxValidBlock ( UInt64 * maxBlock )
 }
 
 
-//
-//	 reportMaxWriteTransfer - Reports maximum write transfer in bytes.
-//																	   [PUBLIC]
-//
-
-IOReturn
-IOBlockStorageServices::reportMaxWriteTransfer ( UInt64		blockSize,
-												 UInt64 * 	max )
-{
-	
-	if ( fMaxWriteBlocks == 0 )
-	{
-		*max = blockSize * 256;
-	}
-	
-	else
-	{
-		*max = blockSize * fMaxWriteBlocks;
-	}
-	
-	return kIOReturnSuccess;
-	
-}
-
-
 //
 //	 reportMediaState - Reports state of media in the device		   [PUBLIC]
 //
@@ -730,10 +706,7 @@ IOBlockStorageServices::attach ( IOService * provider )
 				  fProvider->GetProtocolCharacteristicsDictionary ( ) );
 	setProperty ( kIOPropertyDeviceCharacteristicsKey,
 				  fProvider->GetDeviceCharacteristicsDictionary ( ) );
-	
-	fMaxReadBlocks 	= fProvider->ReportDeviceMaxBlocksReadTransfer ( );
-	fMaxWriteBlocks = fProvider->ReportDeviceMaxBlocksWriteTransfer ( );
-	
+		
 	result = true;
 	
 	
